#!/usr/bin/python
import os
import boto.ec2
from boto.ec2.connection import EC2Connection
from boto.manage.cmdshell import sshclient_from_instance
import time

def _get_account_image_ids(conn):
    available_images = conn.get_all_images(owners='self')
    image_ids = {}
    for image in available_images:
        image_ids[image.name] = image.id
    return image_ids

def _establish_connection(instance, connection):
    #generate files
    ip_address = instance.ip_address
    _generate_files(ip_address)
    #check to see if the vpn is running
    #response = os.system('ping -c 1 ' + ip_address)
    #if the vpn is not up
    #if response != 0:
    #  start the vpn on the cloud machine
    print 'Setting up VPN to IP ' + ip_address
    try:
        ssh_conn = sshclient_from_instance(instance, ssh_key_file='WPI-CMU.pem', user_name='ubuntu')
    except:
        reservations = conn.get_all_reservations()
        for reservation in reservations:
            for running in reservation.instances:
                if running.ip_address == ip_address:
                    instance = running
                    break
        ssh_conn = sshclient_from_instance(instance, ssh_key_file='WPI-CMU.pem', user_name='ubuntu')
    ssh_conn.run('sudo chmod 644 /etc/openvpn/static.key')
    ssh_conn.run('sudo service openvpn restart')
    #  download the vpn key and start the vpn on the local machine
    os.system('scp -i WPI-CMU.pem ubuntu@'+ip_address+':/home/ubuntu/cloudsim/deploy/openvpn.key openvpn.key')
    os.system('sudo ./start_vpn.bash')
    #loop accepting commands from user
    print 'connected to machine running at IP address ' + ip_address
    print 'hostname: ' + instance.public_dns_name
    print 'to save an image of the machine, enter save_ami desired_image_name'
    print 'to terminate the instance, enter terminate'
    print 'to exit, enter exit'
    while True:
        command_input = raw_input('>>')
        command = command_input.split(' ', 1)
        if command[0].upper() == 'EXIT':
            break
        elif command[0].upper() == 'TERMINATE':
            print 'terminating machine and killing the vpn'
            os.system('sudo killall openvpn')
            instance.terminate()
            break
        elif command[0].upper() == 'SAVE_AMI':
            print 'saving an image, the machine will not be usable while this happens'
            connection.create_image(instance.id, command[1])
        else:
            print 'invalid command'

def _generate_files(instance_ip):
    open_vpn_config = open("openvpn.config", "w+")
    open_vpn_config.write('''
    remote ''' + instance_ip + '''
    dev tun
    ifconfig 11.8.0.2 11.8.0.1
    secret openvpn.key
    ''')
    open_vpn_config.close()

    remoteroslaunch = open("remoteroslaunch.bash", "w+")
    remoteroslaunch.write('''#!/bin/bash
    #sudo ./start_vpn.bash
    . ros.bash
    rsync --progress --include='*/' --include='*.xml' --include='*.launch' --include='*.world' --include='*.so' --exclude='*' -rave "ssh -i WPI-CMU.pem" ~/drc_workspace/* ubuntu@'''+instance_ip+''':/home/ubuntu/drc_workspace
    ./ssh-sim.bash '-t' "source /usr/share/drcsim/setup.sh ; source /home/ubuntu/drc_workspace/devel/setup.bash ; export DISPLAY=:0 ; export GAZEBO_IP=11.8.0.1 ; export ROS_IP=11.8.0.1 ; ulimit -s unlimited ; roslaunch $@ gzname:=gzserver"
    ''')
    remoteroslaunch.close()
    os.system('chmod +x remoteroslaunch.bash')

    sshsim = open("sshsim.bash", "w+")
    sshsim.write('''#!/bin/bash

    #
    # This command connects you to the simulation
    # It passes a keyfile to the ssh command and logs you in as the
    # default user on the cloud machine

    # ssh -i key-sim.pem ubuntu@'''+instance_ip+'''

    #
    # This commands is similar, but it suppresses prompts and can be invoked
    # from any directory
    #
    DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
    ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -i "$DIR"/WPI-CMU.pem ubuntu@'''+instance_ip+''' $1 $2 $3 $4 $5''')
    sshsim.close()
    os.system('chmod +x sshsim.bash')

hardcoded_access_key_id='XXXXX'
hardcoded_secret_access_key='XXXXX'
region = "us-east-1"
DEFAULT_AMI_ID = 'ami-60075f08'
ami_id = DEFAULT_AMI_ID

conn = boto.ec2.connect_to_region(region,
    aws_access_key_id=hardcoded_access_key_id,
    aws_secret_access_key=hardcoded_secret_access_key)
   

reservations = conn.get_all_reservations()  

instance_ips = []
account_image_ids = _get_account_image_ids(conn)

print 'Welcome! Here are the already-running machines:'
for reservation in reservations:
    for running in reservation.instances:
        instance_ips.append(running.ip_address)
        if running is not None and running.ip_address is not None:
            image_name = conn.get_image(running.image_id).name
            print 'Instance IP ' + running.ip_address + ' is running AMI ' + image_name
#print 'Would you like to use one of the above instances, or start a new machine?'
print 'Enter target IP address to connect to a running instance, or enter NEW for a new machine.'

while True:
    machine = raw_input('>>')
    if machine in instance_ips:
        target_instance = None
        for reservation in reservations:
            for running in reservation.instances:
                if running.ip_address == machine:
                    target_instance = running
                    break
        _establish_connection(target_instance, conn)
        break
    elif machine.upper() == 'NEW':
        print 'Would you like to use the default machine image, or load a saved one?'
        print 'Enter an image name to use a saved image, or enter DEFAULT for the default image.'
        for image_id in account_image_ids:
            print image_id
        while True:
            targetImage = raw_input('>>')
            if targetImage.upper() == 'DEFAULT':
                ami_id = DEFAULT_AMI_ID
                break
            elif targetImage in account_image_ids.keys():
                ami_id = account_image_ids[targetImage]
                break
            else:
                print 'invalid input'
        launched_reservation = conn.run_instances(ami_id, key_name = 'WPI-CMU', instance_type ='g2.2xlarge')
        launched_instance = launched_reservation.instances[0]
        time.sleep(1)
        print 'launching machine, this can take a few minutes...'
        while launched_instance.update() != "running":
            time.sleep(5)
        _establish_connection(launched_instance, conn)
        break
    else:
        print 'Invalid input, please enter NEW or an IP address'

